@@ -1,10 +1,13 @@
 import numpy as np
 import pytest
 
+import astropy
 import astropy.units as u
 from astropy.tests.helper import quantity_allclose, assert_quantity_allclose
 from astropy.coordinates import (SkyCoord, get_body_barycentric, Angle,
-                                 ConvertError, Longitude, CartesianRepresentation)
+                                 ConvertError, Longitude, CartesianRepresentation,
+                                 get_body_barycentric_posvel,
+                                 CartesianDifferential, SphericalDifferential)
 # Versions of Astropy that do not have HeliocentricMeanEcliptic have the same frame
 # with the misleading name HeliocentricTrueEcliptic
 try:
@@ -156,11 +159,11 @@ def test_hgs_hcrs():
 def test_hgs_hgc_roundtrip():
     obstime = "2011-01-01"
 
-    hgsin = HeliographicStonyhurst(lat=0*u.deg, lon=0*u.deg, obstime=obstime)
+    hgsin = HeliographicStonyhurst(lat=10*u.deg, lon=20*u.deg, obstime=obstime)
     hgcout = hgsin.transform_to(HeliographicCarrington(obstime=obstime))
 
     assert_quantity_allclose(hgsin.lat, hgcout.lat)
-    assert_quantity_allclose(hgcout.lon, sun.L0(obstime))
+    assert_quantity_allclose(hgsin.lon + sun.L0(obstime), hgcout.lon)
 
     hgsout = hgcout.transform_to(HeliographicStonyhurst(obstime=obstime))
 
@@ -293,6 +296,19 @@ def test_hpc_to_hcc_same_observer():
     assert_quantity_allclose(hcccoord_out.z, hcccoord_expected.z)
 
 
+def test_hpc_hcc_different_observer_radius():
+    # Tests HPC->HCC with a change in observer at different distances from the Sun
+    observer1 = HeliographicStonyhurst(0*u.deg, 0*u.deg, 1*u.AU)
+    hpc = Helioprojective(0*u.arcsec, 0*u.arcsec, 0.5*u.AU, observer=observer1)
+
+    observer2 = HeliographicStonyhurst(90*u.deg, 0*u.deg, 0.75*u.AU)
+    hcc = hpc.transform_to(Heliocentric(observer=observer2))
+
+    assert_quantity_allclose(hcc.x, -0.5*u.AU)
+    assert_quantity_allclose(hcc.y, 0*u.AU, atol=1e-10*u.AU)
+    assert_quantity_allclose(hcc.z, 0*u.AU, atol=1e-10*u.AU)
+
+
 def test_hgs_hgs():
     # Test HGS loopback transformation
     obstime = Time('2001-01-01')
@@ -315,6 +331,59 @@ def test_hgc_hgc():
     assert_quantity_allclose(new.radius, old.radius, atol=1e-5*u.AU)
 
 
+def test_hcc_hcc():
+    # Test same observer and changing obstime
+    observer = HeliographicStonyhurst(0*u.deg, 0*u.deg, 1*u.AU, obstime='2001-02-01')
+    from_hcc = Heliocentric(0.2*u.AU, 0.3*u.AU, 0.4*u.AU, observer=observer, obstime='2001-01-01')
+    to_hcc = from_hcc.transform_to(Heliocentric(observer=observer, obstime='2001-03-31'))
+
+    # Since the observer is the same, the coordinates should be nearly the same but not exactly
+    # equal due to motion of the origin (the Sun)
+    assert np.all(from_hcc.cartesian.xyz != to_hcc.cartesian.xyz)
+    assert_quantity_allclose(from_hcc.cartesian.xyz, to_hcc.cartesian.xyz, rtol=2e-3)
+
+    # Test changing observer and same obstime
+    observer1 = HeliographicStonyhurst(0*u.deg, 0*u.deg, 1*u.AU, obstime='2001-01-01')
+    observer2 = HeliographicStonyhurst(0*u.deg, 0*u.deg, 1*u.AU, obstime='2001-03-31')
+    from_hcc = Heliocentric(0.2*u.AU, 0.3*u.AU, 0.4*u.AU, observer=observer1, obstime='2001-02-01')
+    to_hcc = from_hcc.transform_to(Heliocentric(observer=observer2, obstime='2001-02-01'))
+
+    # This change in observer is approximately a 90-degree rotation about the Y axis
+    assert_quantity_allclose(to_hcc.x, -from_hcc.z, rtol=2e-3)
+    assert_quantity_allclose(to_hcc.y, from_hcc.y, rtol=2e-3)
+    assert_quantity_allclose(to_hcc.z, from_hcc.x, rtol=2e-3)
+
+
+def test_hcc_hgs_observer_mismatch():
+    # Test whether the transformation gives the same answer regardless of what obstime the observer
+    # coordinate is represented in
+    observer1 = HeliographicStonyhurst(0*u.deg, 0*u.deg, 1*u.AU, obstime='2001-01-01')
+    observer2 = observer1.transform_to(HeliographicStonyhurst(obstime='2001-03-31'))
+
+    hcc1 = Heliocentric(0.2*u.AU, 0.3*u.AU, 0.4*u.AU, observer=observer1, obstime=observer1.obstime)
+    hgs1 = hcc1.transform_to(HeliographicStonyhurst(obstime=hcc1.obstime))
+
+    hcc2 = Heliocentric(0.2*u.AU, 0.3*u.AU, 0.4*u.AU, observer=observer2, obstime=observer1.obstime)
+    hgs2 = hcc2.transform_to(HeliographicStonyhurst(obstime=hcc2.obstime))
+
+    assert_quantity_allclose(hgs1.lon, hgs2.lon)
+    assert_quantity_allclose(hgs1.lat, hgs2.lat)
+    assert_quantity_allclose(hgs1.radius, hgs2.radius)
+
+
+def test_hgs_hcc_observer_mismatch():
+    # Test whether the transformation gives the same answer regardless of what obstime the observer
+    # coordinate is represented in
+    observer1 = HeliographicStonyhurst(0*u.deg, 0*u.deg, 1*u.AU, obstime='2001-01-01')
+    observer2 = observer1.transform_to(HeliographicStonyhurst(obstime='2001-03-31'))
+
+    hgs = HeliographicStonyhurst(20*u.deg, 40*u.deg, 0.5*u.AU, obstime=observer1.obstime)
+    hcc1 = hgs.transform_to(Heliocentric(observer=observer1, obstime=hgs.obstime))
+    hcc2 = hgs.transform_to(Heliocentric(observer=observer2, obstime=hgs.obstime))
+
+    assert_quantity_allclose(hcc1.cartesian.xyz, hcc2.cartesian.xyz)
+
+
 def test_hgs_hcrs_sunspice():
     # Compare our HGS->HCRS transformation against SunSPICE by transforming beyond it
     # "HEQ" is another name for HEEQ, which is equivalent to Heliographic Stonyhurst
@@ -384,3 +453,74 @@ def test_hgs_hcc_sunspice():
     assert_quantity_allclose(new.x, 800000.00*u.km, atol=1e-2*u.km)
     assert_quantity_allclose(new.y, 908797.89*u.km, atol=1e-2*u.km)
     assert_quantity_allclose(new.z, 688539.32*u.km, atol=1e-2*u.km)
+
+
+def test_hpc_hgs_implicit_hcc():
+    # An HPC->HGS transformation should give the same answer whether the transformation step
+    #   through HCC is implicit or explicit
+    start = SkyCoord(0*u.arcsec, 0*u.arcsec, 0.5*u.AU,
+                     frame=Helioprojective(obstime='2019-06-01', observer='earth'))
+    frame = HeliographicStonyhurst(obstime='2019-12-01')
+
+    implicit = start.transform_to(frame)
+    explicit1 = start.transform_to(Heliocentric(obstime=start.obstime, observer='earth')).\
+        transform_to(frame)
+    explicit2 = start.transform_to(Heliocentric(obstime=frame.obstime, observer='earth')).\
+        transform_to(frame)
+
+    assert_quantity_allclose(implicit.separation_3d(explicit1), 0*u.AU, atol=1e-10*u.AU)
+    assert_quantity_allclose(implicit.separation_3d(explicit2), 0*u.AU, atol=1e-10*u.AU)
+
+
+@pytest.mark.skipif(astropy.__version__ < '3.2.0', reason="Not supported by Astropy <3.2")
+def test_velocity_hcrs_hgs():
+    # Obtain the position/velocity of Earth in ICRS
+    obstime = Time(['2019-01-01', '2019-04-01', '2019-07-01', '2019-10-01'])
+    pos, vel = get_body_barycentric_posvel('earth', obstime)
+    loc = pos.with_differentials(vel.represent_as(CartesianDifferential))
+    earth = SkyCoord(loc, frame='icrs', obstime=obstime)
+
+    # The velocity of Earth in HGS should be very close to zero.  The velocity in the HGS Y
+    # direction is slightly further away from zero because there is true latitudinal motion.
+    new = earth.heliographic_stonyhurst
+    assert_quantity_allclose(new.velocity.d_x, 0*u.km/u.s, atol=1e-15*u.km/u.s)
+    assert_quantity_allclose(new.velocity.d_y, 0*u.km/u.s, atol=1e-14*u.km/u.s)
+    assert_quantity_allclose(new.velocity.d_x, 0*u.km/u.s, atol=1e-15*u.km/u.s)
+
+    # Test the loopback to ICRS
+    newer = new.icrs
+    assert_quantity_allclose(newer.velocity.d_x, vel.x)
+    assert_quantity_allclose(newer.velocity.d_y, vel.y)
+    assert_quantity_allclose(newer.velocity.d_z, vel.z)
+
+
+def test_velocity_hgs_hgc():
+    # Construct a simple HGS coordinate with zero velocity
+    obstime = Time(['2019-01-01', '2019-04-01', '2019-07-01', '2019-10-01'])
+    pos = CartesianRepresentation(1, 0, 0)*u.AU
+    vel = CartesianDifferential(0, 0, 0)*u.km/u.s
+    loc = (pos.with_differentials(vel))._apply('repeat', obstime.size)
+    coord = SkyCoord(HeliographicStonyhurst(loc, obstime=obstime))
+
+    # The induced velocity in HGC should be entirely longitudinal, and approximately equal to one
+    # full rotation every mean synodic period (27.2753 days)
+    new = coord.heliographic_carrington
+    new_vel = new.data.differentials['s'].represent_as(SphericalDifferential, new.data)
+    assert_quantity_allclose(new_vel.d_lon, -360*u.deg / (27.27253*u.day), rtol=1e-2)
+    assert_quantity_allclose(new_vel.d_lat, 0*u.deg/u.s)
+    assert_quantity_allclose(new_vel.d_distance, 0*u.km/u.s, atol=1e-7*u.km/u.s)
+
+
+def test_array_obstime():
+    # Validate that you can transform from an array of obstimes to no obstimes,
+    # or different obstimes.
+    a = SkyCoord([10]*2, [10]*2, unit=u.deg,
+                 observer="earth",
+                 obstime=["2019-01-01", "2019-01-02"],
+                 frame="heliographic_carrington")
+
+    t = a.transform_to(Helioprojective)
+    assert isinstance(t.frame, Helioprojective)
+
+    t2 = a.transform_to(Helioprojective(obstime=["2019-01-03", "2019-01-04"]))
+    assert isinstance(t2.frame, Helioprojective)
