@@ -18,13 +18,14 @@ from copy import deepcopy
 import numpy as np
 
 import astropy.units as u
-from astropy.coordinates import ICRS, HCRS, ConvertError, BaseCoordinateFrame, get_body_barycentric
+from astropy.coordinates import (ICRS, HCRS, ConvertError, BaseCoordinateFrame,
+                                 get_body_barycentric, get_body_barycentric_posvel)
 from astropy.coordinates.baseframe import frame_transform_graph
 from astropy.coordinates.representation import (CartesianRepresentation, SphericalRepresentation,
-                                                UnitSphericalRepresentation)
-from astropy.coordinates.transformations import (FunctionTransform, DynamicMatrixTransform,
-                                                 AffineTransform)
-from astropy.coordinates.matrix_utilities import matrix_product, rotation_matrix, matrix_transpose
+                                                UnitSphericalRepresentation, CartesianDifferential)
+from astropy.coordinates.transformations import (FunctionTransform, AffineTransform,
+                                                 FunctionTransformWithFiniteDifference)
+from astropy.coordinates.matrix_utilities import rotation_matrix, matrix_transpose
 
 from sunpy.sun import constants
 
@@ -46,19 +47,6 @@ __all__ = ['hgs_to_hgc', 'hgc_to_hgs', 'hcc_to_hpc',
            'hgs_to_hgs', 'hgc_to_hgc', 'hcc_to_hcc']
 
 
-def _carrington_offset(obstime):
-    """
-    Calculate the HG Longitude offest based on a time
-    """
-    if obstime is None:
-        raise ValueError("To perform this transformation the coordinate"
-                         " Frame needs a obstime Attribute")
-
-    # Import here to avoid a circular import
-    from .sun import L0
-    return L0(obstime)
-
-
 def _observers_are_equal(obs_1, obs_2, string_ok=False):
     if string_ok:
         if obs_1 == obs_2:
@@ -66,9 +54,10 @@ def _observers_are_equal(obs_1, obs_2, string_ok=False):
     if not (isinstance(obs_1, BaseCoordinateFrame) and isinstance(obs_2, BaseCoordinateFrame)):
         raise ValueError("To compare two observers, both must be instances of BaseCoordinateFrame. "
                          "Cannot compare two observers {} and {}.".format(obs_1, obs_2))
-    return (u.allclose(obs_1.lat, obs_2.lat) and
-            u.allclose(obs_1.lon, obs_2.lon) and
-            u.allclose(obs_1.radius, obs_2.radius))
+    return np.atleast_1d((u.allclose(obs_1.lat, obs_2.lat) and
+                          u.allclose(obs_1.lon, obs_2.lon) and
+                          u.allclose(obs_1.radius, obs_2.radius) and
+                          obs_1.obstime == obs_2.obstime)).all()
 
 
 # =============================================================================
@@ -76,108 +65,158 @@ def _observers_are_equal(obs_1, obs_2, string_ok=False):
 # =============================================================================
 
 
-@frame_transform_graph.transform(FunctionTransform, HeliographicStonyhurst,
-                                 HeliographicCarrington)
+def _transform_obstime(frame, obstime):
+    """
+    Transform a frame to a new obstime using the appropriate loopback transformation.
+    If the new obstime is None, no transformation is performed.
+    If the frame's obstime is None, the frame is copied with the new obstime.
+    """
+    # If obstime is None or the obstime matches, nothing needs to be done
+    if obstime is None or np.all(frame.obstime == obstime):
+        return frame
+
+    # Transform to the new obstime using the appropriate loopback transformation
+    new_frame = frame.replicate(obstime=obstime)
+    if frame.obstime is not None:
+        return frame.transform_to(new_frame)
+    else:
+        return new_frame
+
+
+def _rotation_matrix_hgs_to_hgc(obstime):
+    """
+    Return the rotation matrix from HGS to HGC at the same observation time
+    """
+    if obstime is None:
+        raise ValueError("To perform this transformation the coordinate"
+                         " Frame needs an obstime Attribute")
+
+    # Import here to avoid a circular import
+    from .sun import L0
+
+    # Rotation is only in longitude, so only around the Z axis
+    return rotation_matrix(-L0(obstime), 'z')
+
+
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 HeliographicStonyhurst, HeliographicCarrington)
 def hgs_to_hgc(hgscoord, hgcframe):
     """
-    Transform from Heliographic Stonyhurst to Heliograpic Carrington.
+    Convert from Heliographic Stonyhurst to Heliographic Carrington.
     """
-    if hgcframe.obstime is None or np.any(hgcframe.obstime != hgscoord.obstime):
-        raise ValueError("Can not transform from Heliographic Stonyhurst to "
-                         "Heliographic Carrington, unless both frames have matching obstime.")
+    # First transform the HGS coord to the HGC obstime
+    int_coord = _transform_obstime(hgscoord, hgcframe.obstime)
 
-    c_lon = hgscoord.spherical.lon + _carrington_offset(hgscoord.obstime).to(u.deg)
-    representation = SphericalRepresentation(c_lon, hgscoord.spherical.lat,
-                                             hgscoord.spherical.distance)
-    hgcframe = hgcframe.__class__(obstime=hgscoord.obstime)
+    # Rotate from HGS to HGC
+    total_matrix = _rotation_matrix_hgs_to_hgc(int_coord.obstime)
+    newrepr = int_coord.cartesian.transform(total_matrix)
 
-    return hgcframe.realize_frame(representation)
+    return hgcframe.realize_frame(newrepr)
 
 
-@frame_transform_graph.transform(FunctionTransform, HeliographicCarrington,
-                                 HeliographicStonyhurst)
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 HeliographicCarrington, HeliographicStonyhurst)
 def hgc_to_hgs(hgccoord, hgsframe):
     """
-    Convert from Heliograpic Carrington to Heliographic Stonyhurst.
+    Convert from Heliographic Carrington to Heliographic Stonyhurst.
     """
-    if hgsframe.obstime is None or np.any(hgsframe.obstime != hgccoord.obstime):
-        raise ValueError("Can not transform from Heliographic Carrington to "
-                         "Heliographic Stonyhurst, unless both frames have matching obstime.")
-    obstime = hgsframe.obstime
-    s_lon = hgccoord.spherical.lon - _carrington_offset(obstime).to(
-        u.deg)
-    representation = SphericalRepresentation(s_lon, hgccoord.spherical.lat,
-                                             hgccoord.spherical.distance)
+    # First transform the HGC coord to the HGS obstime
+    int_coord = _transform_obstime(hgccoord, hgsframe.obstime)
 
-    return hgsframe.realize_frame(representation)
+    # Rotate from HGC to HGS
+    total_matrix = matrix_transpose(_rotation_matrix_hgs_to_hgc(int_coord.obstime))
+    newrepr = int_coord.cartesian.transform(total_matrix)
+
+    return hgsframe.realize_frame(newrepr)
 
 
-@frame_transform_graph.transform(FunctionTransform, Heliocentric,
-                                 Helioprojective)
+def _matrix_hcc_to_hpc():
+    # Returns the transformation matrix that permutes/swaps axes from HCC to HPC
+
+    # HPC spherical coordinates are a left-handed frame with these equivalent Cartesian axes:
+    #   HPC_X = -HCC_Z
+    #   HPC_Y = HCC_X
+    #   HPC_Z = HCC_Y
+    # (HPC_X and HPC_Y are not to be confused with HPC_Tx and HPC_Ty)
+    return np.array([[0, 0, -1],
+                     [1, 0, 0],
+                     [0, 1, 0]])
+
+
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 Heliocentric, Helioprojective)
 def hcc_to_hpc(helioccoord, heliopframe):
     """
-    Convert from Heliocentic Cartesian to Helioprojective Cartesian.
+    Convert from Heliocentric Cartesian to Helioprojective Cartesian.
     """
-    if not _observers_are_equal(helioccoord.observer, heliopframe.observer):
-        heliocframe = Heliocentric(observer=heliopframe.observer)
-        new_helioccoord = helioccoord.transform_to(heliocframe)
-        helioccoord = new_helioccoord
+    # Transform the HPC observer (in HGS) to the HPC obstime in case it's different
+    observer = _transform_obstime(heliopframe.observer, heliopframe.obstime)
 
-    x = helioccoord.x.to(u.m)
-    y = helioccoord.y.to(u.m)
-    z = helioccoord.z.to(u.m)
+    # Loopback transform HCC coord to obstime and observer of HPC frame
+    int_frame = Heliocentric(obstime=heliopframe.obstime, observer=observer)
+    int_coord = helioccoord.transform_to(int_frame)
 
-    # d is calculated as the distance between the points
-    # (x,y,z) and (0,0,D0).
-    distance = np.sqrt(x**2 + y**2 + (helioccoord.observer.radius - z)**2)
+    # Shift the origin from the Sun to the observer
+    distance = int_coord.observer.radius
+    newrepr = int_coord.cartesian - CartesianRepresentation(0*u.m, 0*u.m, distance)
 
-    hpcx = np.rad2deg(np.arctan2(x, helioccoord.observer.radius - z))
-    hpcy = np.rad2deg(np.arcsin(y / distance))
+    # Permute/swap axes from HCC to HPC equivalent Cartesian
+    newrepr = newrepr.transform(_matrix_hcc_to_hpc())
 
-    representation = SphericalRepresentation(hpcx, hpcy,
-                                             distance.to(u.km))
-
-    return heliopframe.realize_frame(representation)
+    # Explicitly represent as spherical because external code (e.g., wcsaxes) expects it
+    return heliopframe.realize_frame(newrepr.represent_as(SphericalRepresentation))
 
 
-@frame_transform_graph.transform(FunctionTransform, Helioprojective,
-                                 Heliocentric)
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 Helioprojective, Heliocentric)
 def hpc_to_hcc(heliopcoord, heliocframe):
     """
     Convert from Helioprojective Cartesian to Heliocentric Cartesian.
     """
-    if not _observers_are_equal(heliopcoord.observer, heliocframe.observer):
-        heliocframe_heliopobs = Heliocentric(observer=heliopcoord.observer)
-        helioccoord_heliopobs = heliopcoord.transform_to(heliocframe_heliopobs)
-        helioccoord = helioccoord_heliopobs.transform_to(heliocframe)
-        return helioccoord
-
     if not isinstance(heliopcoord.observer, BaseCoordinateFrame):
         raise ConvertError("Cannot transform helioprojective coordinates to "
                            "heliocentric coordinates for observer '{}' "
                            "without `obstime` being specified.".format(heliopcoord.observer))
 
     heliopcoord = heliopcoord.calculate_distance()
-    x = np.deg2rad(heliopcoord.Tx)
-    y = np.deg2rad(heliopcoord.Ty)
 
-    cosx = np.cos(x)
-    sinx = np.sin(x)
-    cosy = np.cos(y)
-    siny = np.sin(y)
+    # Permute/swap axes from HPC equivalent Cartesian to HCC
+    newrepr = heliopcoord.cartesian.transform(matrix_transpose(_matrix_hcc_to_hpc()))
 
-    rx = (heliopcoord.distance.to(u.m)) * cosy * sinx
-    ry = (heliopcoord.distance.to(u.m)) * siny
-    rz = (heliopcoord.observer.radius.to(u.m)) - (
-        heliopcoord.distance.to(u.m)) * cosy * cosx
+    # Transform the HPC observer (in HGS) to the HPC obstime in case it's different
+    observer = _transform_obstime(heliopcoord.observer, heliopcoord.obstime)
 
-    representation = CartesianRepresentation(
-        rx.to(u.km), ry.to(u.km), rz.to(u.km))
-    return heliocframe.realize_frame(representation)
+    # Shift the origin from the observer to the Sun
+    distance = observer.radius
+    newrepr += CartesianRepresentation(0*u.m, 0*u.m, distance)
+
+    # Complete the conversion of HPC to HCC at the obstime and observer of the HPC coord
+    int_coord = Heliocentric(newrepr, obstime=heliopcoord.obstime, observer=observer)
+
+    # Loopback transform HCC as needed
+    return int_coord.transform_to(heliocframe)
 
 
-@frame_transform_graph.transform(FunctionTransform, Heliocentric,
-                                 HeliographicStonyhurst)
+def _rotation_matrix_hcc_to_hgs(longitude, latitude):
+    # Returns the rotation matrix from HCC to HGS based on the observer longitude and latitude
+
+    # Permute the axes of HCC to match HGS Cartesian equivalent
+    #   HGS_X = HCC_Z
+    #   HGS_Y = HCC_X
+    #   HGS_Z = HCC_Y
+    axes_matrix = np.array([[0, 0, 1],
+                            [1, 0, 0],
+                            [0, 1, 0]])
+
+    # Rotate in latitude and longitude (sign difference because of direction difference)
+    lat_matrix = rotation_matrix(latitude, 'y')
+    lon_matrix = rotation_matrix(-longitude, 'z')
+
+    return lon_matrix @ lat_matrix @ axes_matrix
+
+
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 Heliocentric, HeliographicStonyhurst)
 def hcc_to_hgs(helioccoord, heliogframe):
     """
     Convert from Heliocentric Cartesian to Heliographic Stonyhurst.
@@ -187,92 +226,65 @@ def hcc_to_hgs(helioccoord, heliogframe):
                            "heliographic coordinates for observer '{}' "
                            "without `obstime` being specified.".format(helioccoord.observer))
 
-    x = helioccoord.x.to(u.m)
-    y = helioccoord.y.to(u.m)
-    z = helioccoord.z.to(u.m)
+    # Transform the HCC observer (in HGS) to the HCC obstime in case it's different
+    hcc_observer_at_hcc_obstime = _transform_obstime(helioccoord.observer, helioccoord.obstime)
 
-    l0_rad = helioccoord.observer.lon
-    b0_deg = helioccoord.observer.lat
+    total_matrix = _rotation_matrix_hcc_to_hgs(hcc_observer_at_hcc_obstime.lon,
+                                               hcc_observer_at_hcc_obstime.lat)
 
-    cosb = np.cos(np.deg2rad(b0_deg))
-    sinb = np.sin(np.deg2rad(b0_deg))
+    # Transform from HCC to HGS at the HCC obstime
+    newrepr = helioccoord.cartesian.transform(total_matrix)
+    int_coord = HeliographicStonyhurst(newrepr, obstime=helioccoord.obstime)
 
-    hecr = np.sqrt(x**2 + y**2 + z**2)
-    hgln = np.arctan2(x, z * cosb - y * sinb) + l0_rad
-    hglt = np.arcsin((y * cosb + z * sinb) / hecr)
-
-    representation = SphericalRepresentation(
-        np.rad2deg(hgln), np.rad2deg(hglt), hecr.to(u.km))
-    return heliogframe.realize_frame(representation)
+    # Loopback transform HGS if there is a change in obstime
+    return _transform_obstime(int_coord, heliogframe.obstime)
 
 
-@frame_transform_graph.transform(FunctionTransform, HeliographicStonyhurst,
-                                 Heliocentric)
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 HeliographicStonyhurst, Heliocentric)
 def hgs_to_hcc(heliogcoord, heliocframe):
     """
     Convert from Heliographic Stonyhurst to Heliocentric Cartesian.
     """
-    hglon = heliogcoord.spherical.lon
-    hglat = heliogcoord.spherical.lat
-    r = heliogcoord.spherical.distance
-    if r.unit is u.one and u.allclose(r, 1*u.one):
-        r = np.ones_like(r)
-        r *= RSUN_METERS
-
     if not isinstance(heliocframe.observer, BaseCoordinateFrame):
         raise ConvertError("Cannot transform heliographic coordinates to "
                            "heliocentric coordinates for observer '{}' "
                            "without `obstime` being specified.".format(heliocframe.observer))
 
-    l0_rad = heliocframe.observer.lon.to(u.rad)
-    b0_deg = heliocframe.observer.lat
+    # Loopback transform HGS if there is a change in obstime
+    int_coord = _transform_obstime(heliogcoord, heliocframe.obstime)
 
-    lon = np.deg2rad(hglon)
-    lat = np.deg2rad(hglat)
+    # Transform the HCC observer (in HGS) to the HCC obstime in case it's different
+    hcc_observer_at_hcc_obstime = _transform_obstime(heliocframe.observer, heliocframe.obstime)
 
-    cosb = np.cos(b0_deg.to(u.rad))
-    sinb = np.sin(b0_deg.to(u.rad))
+    total_matrix = matrix_transpose(_rotation_matrix_hcc_to_hgs(hcc_observer_at_hcc_obstime.lon,
+                                                                hcc_observer_at_hcc_obstime.lat))
 
-    lon = lon - l0_rad
-
-    cosx = np.cos(lon)
-    sinx = np.sin(lon)
-    cosy = np.cos(lat)
-    siny = np.sin(lat)
-
-    x = r * cosy * sinx
-    y = r * (siny * cosb - cosy * cosx * sinb)
-    zz = r * (siny * sinb + cosy * cosx * cosb)
-
-    representation = CartesianRepresentation(
-        x.to(u.km), y.to(u.km), zz.to(u.km))
-
-    return heliocframe.realize_frame(representation)
+    # Transform from HGS to HCC at the same obstime
+    newrepr = int_coord.cartesian.transform(total_matrix)
+    return heliocframe.realize_frame(newrepr)
 
 
-@frame_transform_graph.transform(FunctionTransform, Helioprojective,
-                                 Helioprojective)
-def hpc_to_hpc(heliopcoord, heliopframe):
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 Helioprojective, Helioprojective)
+def hpc_to_hpc(from_coo, to_frame):
     """
     This converts from HPC to HPC, with different observer location parameters.
     It does this by transforming through HGS.
     """
-    if (heliopcoord.observer == heliopframe.observer or
-        (u.allclose(heliopcoord.observer.lat, heliopframe.observer.lat) and
-         u.allclose(heliopcoord.observer.lon, heliopframe.observer.lon) and
-         u.allclose(heliopcoord.observer.radius, heliopframe.observer.radius))):
-        return heliopframe.realize_frame(heliopcoord._data)
+    if _observers_are_equal(from_coo.observer, to_frame.observer, string_ok=True) and \
+       np.all(from_coo.obstime == to_frame.obstime):
+        return to_frame.realize_frame(from_coo.data)
 
-    if not isinstance(heliopframe.observer, BaseCoordinateFrame):
+    if not isinstance(to_frame.observer, BaseCoordinateFrame):
         raise ConvertError("Cannot transform between helioprojective frames "
-                           "without `obstime` being specified for observer {}.".format(heliopframe.observer))
-    if not isinstance(heliopcoord.observer, BaseCoordinateFrame):
+                           "without `obstime` being specified for observer {}.".format(to_frame.observer))
+    if not isinstance(from_coo.observer, BaseCoordinateFrame):
         raise ConvertError("Cannot transform between helioprojective frames "
-                           "without `obstime` being specified for observer {}.".format(heliopcoord.observer))
+                           "without `obstime` being specified for observer {}.".format(from_coo.observer))
 
-    hgs = heliopcoord.transform_to(HeliographicStonyhurst)
-    hgs.observer = heliopframe.observer
-    hpc = hgs.transform_to(heliopframe)
+    hgs = from_coo.transform_to(HeliographicStonyhurst(obstime=to_frame.obstime))
+    hpc = hgs.transform_to(to_frame)
 
     return hpc
 
@@ -321,10 +333,19 @@ def hcrs_to_hgs(hcrscoord, hgsframe):
         raise ValueError("To perform this transformation the coordinate"
                          " Frame needs an obstime Attribute")
 
+    # Check whether differentials are involved on either end
+    has_differentials = ((hcrscoord._data is not None and hcrscoord.data.differentials) or
+                         (hgsframe._data is not None and hgsframe.data.differentials))
+
     # Determine the Sun-Earth vector in ICRS
     # Since HCRS is ICRS with an origin shift, this is also the Sun-Earth vector in HCRS
-    sun_pos_icrs = get_body_barycentric('sun', hgsframe.obstime)
-    earth_pos_icrs = get_body_barycentric('earth', hgsframe.obstime)
+    # If differentials exist, also obtain Sun and Earth velocities
+    if has_differentials:
+        sun_pos_icrs, sun_vel = get_body_barycentric_posvel('sun', hgsframe.obstime)
+        earth_pos_icrs, earth_vel = get_body_barycentric_posvel('earth', hgsframe.obstime)
+    else:
+        sun_pos_icrs = get_body_barycentric('sun', hgsframe.obstime)
+        earth_pos_icrs = get_body_barycentric('earth', hgsframe.obstime)
     sun_earth = earth_pos_icrs - sun_pos_icrs
 
     # De-tilt the Sun-Earth vector to the frame with the Sun's rotation axis parallel to the Z axis
@@ -342,17 +363,22 @@ def hcrs_to_hgs(hcrscoord, hgsframe):
         rot_matrix_list = [_make_rotation_matrix_from_reprs(vect, x_axis) for vect in hgs_x_axis_detilt]
         rot_matrix = np.stack(rot_matrix_list)
 
-    total_matrix = matrix_product(rot_matrix, _SUN_DETILT_MATRIX)
+    total_matrix = rot_matrix @ _SUN_DETILT_MATRIX
 
     # All of the above is calculated for the HGS observation time
     # If the HCRS observation time is different, calculate the translation in origin
     if np.any(hcrscoord.obstime != hgsframe.obstime):
         sun_pos_old_icrs = get_body_barycentric('sun', hcrscoord.obstime)
         offset_icrf = sun_pos_icrs - sun_pos_old_icrs
-        offset = offset_icrf.transform(total_matrix)
     else:
-        offset = CartesianRepresentation(0, 0, 0)*u.m
+        offset_icrf = sun_pos_icrs * 0  # preserves obstime shape
 
+    # Add velocity if needed (at the HGS observation time)
+    if has_differentials:
+        vel_icrf = (sun_vel - earth_vel).represent_as(CartesianDifferential)
+        offset_icrf = offset_icrf.with_differentials(vel_icrf)
+
+    offset = offset_icrf.transform(total_matrix)
     return total_matrix, offset
 
 
@@ -366,12 +392,20 @@ def hgs_to_hcrs(hgscoord, hcrsframe):
 
     # Invert the transformation to get the HGS->HCRS transformation
     reverse_matrix = matrix_transpose(total_matrix)
-    reverse_offset = (-offset).transform(reverse_matrix)
+    # If differentials exist, properly negate the velocity
+    if offset.differentials:
+        pos = -offset.without_differentials()
+        vel = -offset.differentials['s']
+        offset = pos.with_differentials(vel)
+    else:
+        offset = -offset
+    reverse_offset = offset.transform(reverse_matrix)
 
     return reverse_matrix, reverse_offset
 
 
-@frame_transform_graph.transform(FunctionTransform, HeliographicStonyhurst, HeliographicStonyhurst)
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 HeliographicStonyhurst, HeliographicStonyhurst)
 def hgs_to_hgs(from_coo, to_frame):
     """
     Convert between two Heliographic Stonyhurst frames.
@@ -379,10 +413,11 @@ def hgs_to_hgs(from_coo, to_frame):
     if np.all(from_coo.obstime == to_frame.obstime):
         return to_frame.realize_frame(from_coo.data)
     else:
-        return from_coo.transform_to(HCRS).transform_to(to_frame)
+        return from_coo.transform_to(HCRS(obstime=from_coo.obstime)).transform_to(to_frame)
 
 
-@frame_transform_graph.transform(FunctionTransform, HeliographicCarrington, HeliographicCarrington)
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 HeliographicCarrington, HeliographicCarrington)
 def hgc_to_hgc(from_coo, to_frame):
     """
     Convert between two Heliographic Carrington frames.
@@ -391,22 +426,23 @@ def hgc_to_hgc(from_coo, to_frame):
         return to_frame.realize_frame(from_coo.data)
     else:
         return from_coo.transform_to(HeliographicStonyhurst(obstime=from_coo.obstime)).\
-               transform_to(HeliographicStonyhurst(obstime=to_frame.obstime)).transform_to(to_frame)
+               transform_to(to_frame)
 
 
-@frame_transform_graph.transform(FunctionTransform, Heliocentric, Heliocentric)
-def hcc_to_hcc(hcccoord, hccframe):
+@frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
+                                 Heliocentric, Heliocentric)
+def hcc_to_hcc(from_coo, to_frame):
     """
-    Convert from  Heliocentric to Heliocentric
+    Convert between two Heliocentric frames.
     """
-    if _observers_are_equal(hcccoord.observer, hccframe.observer, string_ok=True):
-        return hccframe.realize_frame(hcccoord._data)
+    if _observers_are_equal(from_coo.observer, to_frame.observer, string_ok=True) and \
+       np.all(from_coo.obstime == to_frame.obstime):
+        return to_frame.realize_frame(from_coo.data)
 
-    hgscoord = hcccoord.transform_to(HeliographicStonyhurst)
-    hgscoord.observer = hccframe.observer
-    hcccoord = hgscoord.transform_to(hccframe)
+    # Convert through HGS
+    hgscoord = from_coo.transform_to(HeliographicStonyhurst(obstime=to_frame.obstime))
 
-    return hcccoord
+    return hgscoord.transform_to(to_frame)
 
 
 def _make_sunpy_graph():
@@ -499,6 +535,13 @@ def _tweak_graph(docstr):
     for frame in sunpy_frames:
         output = output.replace(frame + ' [', frame + ' [fillcolor=white ')
 
+    # Set the rank direction to be left->right (as opposed to top->bottom)
+    # Force nodes for ICRS, HCRS, and "Other frames in Astropy" to be at the same rank
+    output = output.replace('        overlap=false',
+                            '        overlap=false\n'
+                            '        rankdir=LR\n'
+                            '        {rank=same; ICRS; HCRS; Astropy}')
+
     output = output.replace('<ul>\n\n',
                             '<ul>\n\n' +
                             _add_legend_row('SunPy frames', 'white') +
