@@ -80,7 +80,7 @@ def read(filepath, hdus=None, memmap=None, **kwargs):
             try:
                 pairs.append(HDPair(hdu.data, header))
             except (KeyError, ValueError) as e:
-                message = "Error when reading HDU {}. Skipping.\n".format(i)
+                message = f"Error when reading HDU {i}. Skipping.\n"
                 for line in traceback.format_tb(sys.exc_info()[2]):
                     message += line
                     message += '\n'
@@ -160,6 +160,9 @@ def write(fname, data, header, hdu_type=None, **kwargs):
     hdu_type: `None` or `~astropy.io.fits.CompImageHDU`
         `None` will return a normal FITS files.
         `~astropy.io.fits.CompImageHDU` will rice compress the FITS file.
+    kwargs :
+        Additional keyword arguments are given to
+        `~astropy.io.fits.HDUList.writeto`.
     """
     # Copy header so the one in memory is left alone while changing it for
     # write.
