@@ -175,7 +175,7 @@ class GenericMap(NDData):
             warnings.warn("This file contains more than 2 dimensions. "
                           "Data will be truncated to the first two dimensions.", SunpyUserWarning)
 
-        super(GenericMap, self).__init__(data, meta=header, **kwargs)
+        super().__init__(data, meta=header, **kwargs)
 
         # Correct possibly missing meta keywords
         self._fix_date()
@@ -275,7 +275,6 @@ class GenericMap(NDData):
         w2.wcs.pc = self.rotation_matrix
         w2.wcs.cunit = self.spatial_units
         w2.wcs.dateobs = self.date.iso
-        w2.heliographic_observer = self.observer_coordinate
         w2.rsun = self.rsun_meters
 
         # Astropy WCS does not understand the SOHO default of "solar-x" and
@@ -288,6 +287,13 @@ class GenericMap(NDData):
         if w2.wcs.ctype[1].lower() in ("solar-y", "solar_y"):
             w2.wcs.ctype[1] = 'HPLT-TAN'
 
+        # GenericMap.coordinate_frame is implemented using this method, so we
+        # need to do this only based on .meta.
+        ctypes = {c[:4] for c in w2.wcs.ctype}
+        # Check that the ctypes contains one of these two pairs of axes.
+        if {'HPLN', 'HPLT'} <= ctypes or {'SOLX', 'SOLY'} <= ctypes:
+            w2.heliographic_observer = self.observer_coordinate
+
         return w2
 
     @property
@@ -624,7 +630,7 @@ class GenericMap(NDData):
 
     @property
     def dsun(self):
-        """The observer distance from the Sun."""
+        """The observer distance from the center of the Sun."""
         return self.observer_coordinate.radius.to('m')
 
     @property
@@ -877,15 +883,14 @@ class GenericMap(NDData):
         ----------
         filepath : str
             Location to save file to.
-
         filetype : str
             'auto' or any supported file extension.
-
-        Keywords
-        --------
-        hdu_type: `None`, `~fits.CompImageHDU`
-            `None` will return a normal FITS files.
+        hdu_type: None, `~fits.CompImageHDU`
+            `None` will return a normal FITS file.
             `~fits.CompImageHDU` will rice compress the FITS file.
+        kwargs :
+            Any additional keyword arguments are passed to
+            `~sunpy.io.write_file`.
         """
         io.write_file(filepath, self.data, self.meta, filetype=filetype,
                       **kwargs)
@@ -1036,9 +1041,9 @@ class GenericMap(NDData):
                 equivalent = angle.unit.is_equivalent(u.deg)
 
                 if not equivalent:
-                    raise u.UnitsError("Argument '{0}' to function '{1}'"
+                    raise u.UnitsError("Argument '{}' to function '{}'"
                                        " must be in units convertable to"
-                                       " '{2}'.".format('angle', 'rotate',
+                                       " '{}'.".format('angle', 'rotate',
                                                         u.deg.to_string()))
 
             # Either there is no .unit or no .is_equivalent
@@ -1047,7 +1052,7 @@ class GenericMap(NDData):
                     error_msg = "a 'unit' attribute without an 'is_equivalent' method"
                 else:
                     error_msg = "no 'unit' attribute"
-                raise TypeError("Argument '{0}' to function '{1}' has {2}. "
+                raise TypeError("Argument '{}' to function '{}' has {}. "
                                 "You may want to pass in an astropy Quantity instead."
                                 .format('angle', 'rotate', error_msg))
 
