@@ -8,6 +8,8 @@ This module provides the `Fido
 `Fido.fetch <sunpy.net.fido_factory.UnifiedDownloaderFactory.fetch>`.
 
 """
+import os
+from pathlib import Path
 from collections.abc import Sequence
 
 from parfive import Downloader, Results
@@ -185,7 +187,7 @@ class UnifiedResponse(Sequence):
         return ret
 
     def __repr__(self):
-        ret = super(UnifiedResponse, self).__repr__()
+        ret = super().__repr__()
         ret += '\n' + str(self)
 
         return ret
@@ -346,6 +348,11 @@ class UnifiedDownloaderFactory(BasicRegistrationFactory):
         >>> filepaths = Fido.fetch(filepaths)  # doctest: +SKIP
 
         """
+        if path is not None:
+            exists = list(filter(lambda p: p.exists(), Path(path).parents))
+            if not os.access(exists[0], os.W_OK):
+                raise PermissionError('You do not have permission to write'
+                                      f' to the directory {exists[0]}.')
 
         if "wait" in kwargs:
             raise ValueError("wait is not a valid keyword argument to Fido.fetch.")
@@ -391,7 +398,7 @@ class UnifiedDownloaderFactory(BasicRegistrationFactory):
         return results
 
     def __call__(self, *args, **kwargs):
-        raise TypeError("'{}' object is not callable".format(self.__class__.__name__))
+        raise TypeError(f"'{self.__class__.__name__}' object is not callable")
 
     def _check_registered_widgets(self, *args):
         """Factory helper function"""
